package enums

import (
	"errors"

	"github.com/mailru/easyjson/jlexer"
	"github.com/mailru/easyjson/jwriter"
)

type UserType string

const (
	UserTypeAnon  UserType  = "ROLE_ANONYMOUS"
	UserTypeUser  UserType  = "ROLE_USER"
	UserTypeAdmin UserType = "ROLE_ADMIN"
)

func UserTypeNameOf(value string) (UserType, error) {
	switch value {
	case "ROLE_USER":
		return UserTypeUser, nil
	case "ROLE_ADMIN":
		return UserTypeAdmin, nil
	}
	return UserTypeAnon, errors.New("unable to find UserType")
}

func (status UserType) Name() string {
	return string(status)
}

func (status UserType) String() string {
	switch status {
	case UserTypeUser:
		return "사용자"
	case UserTypeAdmin:
		return "관리자"
	}
	return "로그인 안함"
}

func (status UserType) MarshalEasyJSON(w *jwriter.Writer) {
	w.String(string(status))
}
func (status *UserType) UnmarshalEasyJSON(w *jlexer.Lexer) {
	*status, _ = UserTypeNameOf(w.String())
}
