package session

import (
	"amuz.es/gogs/infra/changer/enums"
	"amuz.es/gogs/infra/changer/http/iface"
	"amuz.es/gogs/infra/changer/util"
	"amuz.es/gogs/infra/changer/util/crypt"
	"amuz.es/gogs/infra/changer/util/ldap"
	"github.com/gin-gonic/gin"
	"github.com/kataras/go-errors"
)

var (
	logger        = util.NewLogger("session")
	cookieName    string
	cookieCryptor crypt.AesCryptor
	Authenticator ldap.LdapAuthenticator
)

func New(ldapConfig *util.LdapConfig, sessionConfig *util.SessionConfig) gin.HandlerFunc {
	Authenticator = ldap.NewLdapAuthenticator(
		ldapConfig.Host, ldapConfig.Port,
		ldapConfig.SecurityProtocol, ldapConfig.SkipVerify,
		"", "",
		ldapConfig.BaseDN,
		ldapConfig.UserDN,
		ldapConfig.AttributeUsername,
		ldapConfig.AttributeName,
		ldapConfig.AttributeSurname,
		ldapConfig.AttributeMail,
		ldapConfig.Filter,
		ldapConfig.AdminFilter,
	)
	cookieName = sessionConfig.CookieName
	cookieCryptor = crypt.NewAesCryptor([]byte(sessionConfig.Key))

	return func(c *gin.Context) {
		var (
			err    error
			detail = &userDetail{
				role: &roleAnonymous,
			}
		)
		defer func() {
			c.Set("UserDetail", detail)
			// if err != nil {
			// logger.Debug("failed to TokenAuthFunc ", err)
			// }
			c.Next()
		}()

		var cookie string
		if cookie, err = c.Cookie(cookieName); err != nil {
			return
		}


		var decryptedData []byte

		if decryptedData, err = cookieCryptor.DecryptEncoded(cookie); err != nil {
			return
		}

		var userData iface.UserData
		if err := util.LoadJSON(decryptedData, &userData); err != nil {
			return
		}

		switch userData.Role {
		case enums.UserTypeUser:
			detail.role = &roleUser
		case enums.UserTypeAdmin:
			detail.role = &roleAdmin
		default:
		}
		detail.id = userData.Id
		detail.mail = userData.Mail
		detail.name = userData.Name
		return
	}
}

func AttachSession(c *gin.Context, id, mail, name string, role enums.UserType) {
	userdata := &iface.UserData{
		Id:   id,
		Name: name,
		Mail: mail,
		Role: role,
	}

	var (
		err         error
		encodedData []byte
	)
	if encodedData, err = util.DumpJSONData(userdata); err != nil {
		logger.Error(err)
		return
	}

	var encryptedData string

	if encryptedData, err = cookieCryptor.EncryptEncoded(encodedData); err != nil {
		logger.Error(err)
		return
	}


	c.SetCookie(cookieName, encryptedData, 3600, "", "", false, true)
}
func RemoveSession(c *gin.Context) {
	c.SetCookie(cookieName, "", -1, "", "", false, true)
}

func ExtractUserDetail(c *gin.Context) (UserDetail) {
	if detail, exist := c.Get("UserDetail"); !exist {
		panic(errors.New("UserDetail not exist"))
	} else {
		return detail.(UserDetail)
	}

}