package session

import (
	"amuz.es/gogs/infra/changer/enums"
	"github.com/mikespook/gorbac"
	errs "github.com/pkg/errors"
	"fmt"
)

type ApiPerm gorbac.Permission

type role struct {
	roleType enums.UserType
	gorbac.StdRole
}

type Role interface {
	Permit(gorbac.Permission) bool
	String() string
	Type() enums.UserType
}

func newRole(roleType enums.UserType) role {
	return role{
		roleType,
		*gorbac.NewStdRole(roleType.Name()),
	}
}

func (roleData *role) Type() enums.UserType {
	return roleData.roleType
}

func (roleData *role) String() string {
	return fmt.Sprint(
		"Role(",
		"name:", roleData.roleType.Name(),
		",desc:", roleData.roleType.String(),
		")",
	)
}

var (
	roleAnonymous = newRole(enums.UserTypeAnon)
	roleUser      = newRole(enums.UserTypeUser)
	roleAdmin     = newRole(enums.UserTypeAdmin)
)
var (
	PermChangePasswordPerm = gorbac.NewStdPermission("PERM_CHANGE_PASSWORD")
)

func InitRole() {
	//initAssignPermission(&roleAnonymous, PermActSessCreate)
	initAssignPermission(&roleUser, PermChangePasswordPerm)
	initAssignPermission(&roleAdmin, PermChangePasswordPerm)
}

func initAssignPermission(assignee *role, perm gorbac.Permission) {
	if err := assignee.Assign(perm); err != nil {
		panic(errs.Wrapf(err, "failed to assign permission %s -> role %s", assignee.roleType.Name(), perm.ID()))
	}
}
