package route

import (
	"net/http"
	"strings"

	"amuz.es/gogs/infra/changer/enums"
	"amuz.es/gogs/infra/changer/http/iface"
	"amuz.es/gogs/infra/changer/http/middleware/session"
	"amuz.es/gogs/infra/changer/util"
	"github.com/flosch/pongo2"
	"github.com/gin-gonic/gin"
)

func Login(c *gin.Context) {
	var (
		code int
		body = iface.LoginResponse{}
	)
	defer func() {
		if err := recover(); err != nil {
			panic(err)
		} else {
			util.DumpJSON(c, code, body)
		}
	}()

	var (
		request iface.Login
	)
	code, body.Message = 406, "요청이 올바르지 않습니다."

	// unboxing
	if err := util.BindJSON(c, &request); err != nil {
		logger.Error(err)
		return
	}
	if len(request.RedirectTo) == 0 || strings.Contains(request.RedirectTo, "login") {
		request.RedirectTo = "./"
	}

	code, body.Message = 403, "ID가 잘못되었거나 기존암호가 일치하지 않습니다."

	if session.Authenticator.AuthenticateUser(request.Id, request.Password, false) != nil {
		return
	}

	code, body.Message, body.RedirectTo = 200, "성공!", request.RedirectTo

	var mail, name string
	if userInfo, err := session.Authenticator.GetInfo(request.Id); err != nil {
		return
	} else {
		name = userInfo["username"].(string)
		mail = userInfo["mail"].(string)
	}

	var role enums.UserType = enums.UserTypeAnon
	if fetchedRole, err := session.Authenticator.SearchPermission(request.Id); err != nil {
		logger.Info(err)
		return
	} else {
		role = fetchedRole
	}

	session.AttachSession(c, request.Id, mail, name, role)

}

func Logout(c *gin.Context) {
	session.RemoveSession(c)
	c.Redirect(302, "./")
}

func LoginView(c *gin.Context) {
	context := session.ExtractUserDetail(c)
	modelContext := pongo2.Context{}
	modelContext.Update(context.ToModelContext())
	modelContext["redirectTo"] = c.Query("redirectTo")

	c.HTML(http.StatusOK, "views/login.html", modelContext)
}
