package route

import (
	"net/http"
	"strings"

	"amuz.es/gogs/infra/changer/bind/static"
	assetfs "github.com/elazarl/go-bindata-assetfs"
	"amuz.es/gogs/infra/changer/util"
)
var logger = util.NewLogger("route")

type binaryFileSystem struct {
	fs http.FileSystem
}

func (b *binaryFileSystem) Open(name string) (http.File, error) {
	return b.fs.Open(name)
}

func (b *binaryFileSystem) Exists(prefix string, filepath string) bool {

	if p := strings.TrimPrefix(filepath, prefix); len(p) < len(filepath) {
		if _, err := b.fs.Open(p); err != nil {
			return false
		}
		return true
	}
	return false
}

func Static(root string) *binaryFileSystem {
	fs := &assetfs.AssetFS{Asset: static.Asset, AssetDir: static.AssetDir, AssetInfo: static.AssetInfo, Prefix: root}
	return &binaryFileSystem{
		fs,
	}
}
