package template

import (
	"net/http"

	"amuz.es/gogs/infra/changer/bind/template"
	"github.com/flosch/pongo2"
	"github.com/gin-gonic/gin/render"
)

// RenderOptions is used to configure the renderer.
type RenderOptions struct {
	contentType string
	templateSet *pongo2.TemplateSet
}

// Pongo2Render is a custom Gin template renderer using Pongo2.
type Pongo2Render struct {
	template    *pongo2.Template
	contentType *string
	context     pongo2.Context
}

// Default creates a Pongo2Render instance with default options.
func DefaultRender() *RenderOptions {
	return &RenderOptions{
		contentType: "text/html; charset=utf-8",
		templateSet: pongo2.NewSet("templates", NewMemoryTemplateLoader(template.Asset)),
	}
}

// Instance should return a new Pongo2Render struct per request and prepare
// the template by either loading it from disk or using pongo2's cache.
func (p *RenderOptions) Instance(name string, data interface{}) render.Render {
	template, err := p.templateSet.FromFile(name)
	if err != nil {
		panic(err)
	}
	return &Pongo2Render{
		template:    template,
		contentType: &p.contentType,
		context:     data.(pongo2.Context),
	}
}

// Render should render the template to the response.
func (p *Pongo2Render) Render(w http.ResponseWriter) error {
	writeContentType(w, []string{*p.contentType})
	return p.template.ExecuteWriter(p.context, w)
}

// writeContentType is also in the gin/render package but it has not been made
// pubic so is repeated here, maybe convince the author to make this public.
func writeContentType(w http.ResponseWriter, value []string) {
	header := w.Header()
	if val := header["Content-Type"]; len(val) == 0 {
		header["Content-Type"] = value
	}
}
