package util

import (
	"io/ioutil"

	"gopkg.in/yaml.v2"
)


type LogConfig struct {
	FileName  string
	MaxSizeMb int
	MaxBackup int
	MaxDay    int
}

type SessionConfig struct {
	Key   string
	CookieName   string
	ExpireSeconds int
}
type LdapConfig struct {
	Host              string
	Port              int
	SecurityProtocol  int
	SkipVerify        bool
	BaseDN            string
	UserDN            string
	AttributeUsername string
	AttributeName     string
	AttributeSurname  string
	AttributeMail     string
	AttributePassword string
	Filter            string
	AdminFilter       string
}

type config struct {
	Phase   string
	Bind    string
	Prefix  string
	Ldap    LdapConfig
	Session SessionConfig
	Logging struct {
		Application LogConfig
		Access      LogConfig
	}
}

func (c config) Name() string {
	return name
}

func (c config) Version() string {
	return version
}

func (c config) BuildDate() string {
	return buildDate
}

const name = "Changer"

var (
	buildDate string
	version   string
	Config    = config{}
)

func LoadConfig(path string) error {
	var (
		source []byte
		err    error
	)
	if source, err = ioutil.ReadFile(path); err != nil {
		return err
	}

	if err = yaml.Unmarshal(source, &Config); err != nil {
		return err
	}

	return nil
}
