package crypt

import (
	"bytes"
	"crypto/rand"
	"crypto/sha512"
	"encoding/base64"
	"strconv"

	"golang.org/x/crypto/pbkdf2"
)

/*
import hashlib
from base64 import urlsafe_b64encode as encode
from base64 import urlsafe_b64decode as decode

def makeSecret(password):
salt = os.urandom(4)
h = hashlib.sha1(password)
h.update(salt)
return "{SSHA}" + encode(h.digest() + salt)

def checkPassword(challenge_password, password):
challenge_bytes = decode(challenge_password[6:])
digest = challenge_bytes[:20]
salt = challenge_bytes[20:]
hr = hashlib.sha1(password)
hr.update(salt)
return digest == hr.digest()
*/

var (
	iter       = 20000
	saltLengthByte = 16
	hasher     = sha512.New
)

func MakeSaltedPassword(password string) (string, error) {
	salt := make([]byte, saltLengthByte)
	if _, err := rand.Read(salt); err != nil {
		return "", err
	}

	dk := pbkdf2.Key([]byte(password), salt, iter, sha512.Size, hasher)

	var buffer bytes.Buffer

	buffer.WriteString("{PBKDF2-SHA512}")
	buffer.WriteString(strconv.FormatUint((uint64)(iter), 10))
	buffer.WriteRune('$')

	encoder := base64.NewEncoder(base64.RawStdEncoding, &buffer)

	if _, err := encoder.Write(salt); err != nil {
		return "", err
	}
	if err := encoder.Close(); err != nil {
		return "", err
	}
	buffer.WriteRune('$')

	if _, err := encoder.Write(dk); err != nil {
		return "", err
	}

	if err := encoder.Close(); err != nil {
		return "", err
	}
	return buffer.String(), nil
}
