package util

import (
	"bytes"
	"encoding/json"
	"errors"
	"io"
	"net/http"

	"github.com/gin-gonic/gin"
	"github.com/gin-gonic/gin/binding"
)

var jsonContentType = []string{"application/json; charset=utf-8"}

func writeContentType(w http.ResponseWriter, value []string) {
	header := w.Header()
	if val := header["Content-Type"]; len(val) == 0 {
		header["Content-Type"] = value
	}
}

func DumpJSONHeaderOnly(c *gin.Context, code int) {
	c.Status(code)

	// Encode
	header := c.Writer.Header()
	if val := header["Content-Type"]; len(val) == 0 {
		header["Content-Type"] = jsonContentType
	}
}

func DumpJSON(c *gin.Context, code int, serializable interface{}) {
	c.Status(code)

	// Encode
	header := c.Writer.Header()
	if val := header["Content-Type"]; len(val) == 0 {
		header["Content-Type"] = jsonContentType
	}

	if enc := json.NewEncoder(c.Writer); enc == nil {
		panic(errors.New("empty json encoder"))
	} else if err := enc.Encode(serializable); err != nil {
		panic(err)
	}
}

func DumpJSONData(serializable interface{}) (data []byte,err error){
	var buf bytes.Buffer
	if enc := json.NewEncoder(&buf); enc == nil {
		return nil,errors.New("empty json encoder")
	} else if err := enc.Encode(serializable); err != nil {
		return nil,err
	}else{
		return buf.Bytes(),nil
	}
}

func LoadJSON(data []byte, deserializable interface{}) (err error) {
	if reader := bytes.NewReader(data); reader == nil {
		err = errors.New("empty json data")
	} else {
		err = LoadJSONReader(reader, deserializable)
	}
	return

}

func LoadJSONReader(r io.Reader, deserializable interface{}) (err error) {
	decoder := json.NewDecoder(r)

	return decoder.Decode(deserializable)
}

func BindJSON(c *gin.Context, deserializable interface{}) (err error) {
	defer func() {
		if err != nil {
			c.AbortWithError(400, err).SetType(gin.ErrorTypeBind)
		}
	}()
	if err = LoadJSONReader(c.Request.Body, deserializable); err != nil {
		return
	}

	// validate(obj)

	if binding.Validator == nil {
		return nil
	}
	return binding.Validator.ValidateStruct(deserializable)
}
